package com.cultural.heritage.controller.book;


import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.cultural.heritage.annotation.AuthCheck;
import com.cultural.heritage.common.BaseResponse;
import com.cultural.heritage.common.ErrorCode;
import com.cultural.heritage.common.ResultUtils;
import com.cultural.heritage.constant.UserConstant;
import com.cultural.heritage.exception.BusinessException;
import com.cultural.heritage.exception.ThrowUtils;
import com.cultural.heritage.model.dto.advanceOrder.AdvanceOrderAddRequest;
import com.cultural.heritage.model.dto.advanceOrder.AdvanceOrderQueryRequest;
import com.cultural.heritage.model.dto.order.OrderUpdateRequest;
import com.cultural.heritage.model.dto.snapshot.ContactsSnapshot;
import com.cultural.heritage.model.entity.AdvanceOrder;
import com.cultural.heritage.model.entity.Contacts;
import com.cultural.heritage.model.entity.User;
import com.cultural.heritage.model.vo.advanceorder.AdvanceOrderVO;
import com.cultural.heritage.service.address.ContactsService;
import com.cultural.heritage.service.book.AdvanceOrderService;
import com.cultural.heritage.service.user.UserService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.annotation.Resource;
import jakarta.servlet.http.HttpServletRequest;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

@RestController
@RequestMapping("/advanceOrder")
@Slf4j
@Tag(name = "写真预约订单管理模块")
public class AdvanceOrderController {


    @Resource
    private AdvanceOrderService advanceOrderService;


    @Resource
    private UserService userService;


    @Resource
    private ContactsService contactsService;


    /**
     * 小程序端用户创建写真预约订单
     * @param addAdvanceOrderRequest    写真预约订单添加请求体
     * @return  是否添加成功
     */
    @PostMapping("/add")
    @Operation(summary = "小程序端用户创建写真预约订单", description = "参数：写真预约订单添加请求体, 权限：所有人，方法名：addAdvanceOrder")
    public BaseResponse<Boolean> addAdvanceOrder(@RequestBody AdvanceOrderAddRequest addAdvanceOrderRequest, HttpServletRequest request) {
        if (addAdvanceOrderRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        // 获取用户id
        User loginUser = userService.getLoginUser(request);
        Long userId = loginUser.getId();
        // 获取联系人信息
        Long contactsId = addAdvanceOrderRequest.getContactsId();
        Contacts contacts = contactsService.getById(contactsId);
        ContactsSnapshot contactsSnapshot = new ContactsSnapshot();
        BeanUtils.copyProperties(contacts, contactsSnapshot);
        // 封装写真预约订单类
        AdvanceOrder advanceOrder = new AdvanceOrder();
        BeanUtils.copyProperties(addAdvanceOrderRequest, advanceOrder);
        advanceOrder.setUserId(userId);
        advanceOrder.setContactsSnapshot(contactsSnapshot);
        // 校验
        advanceOrderService.validAdvanceOrder(advanceOrder);
        boolean result = advanceOrderService.save(advanceOrder);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR);
        return ResultUtils.success(true);
    }




    /**
     * Web端管理员查询写真预约订单
     * @param advanceOrderQueryRequest  写真预约订单查询请求体
     * @return  写真预约订单列表
     */
    @PostMapping("/list")
    @Operation(summary = "Web端管理员查询写真预约订单", description = "参数：写真预约订单查询请求体, 权限：管理员(admin, boss)，方法名：listAdvanceOrderByPage")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Page<AdvanceOrderVO>> listAdvanceOrderByPage(@RequestBody AdvanceOrderQueryRequest advanceOrderQueryRequest) {
        if (advanceOrderQueryRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        long current = advanceOrderQueryRequest.getCurrent();
        long pageSize = advanceOrderQueryRequest.getPageSize();
        QueryWrapper<AdvanceOrder> queryWrapper = advanceOrderService.getQueryWrapper(advanceOrderQueryRequest);
        Page<AdvanceOrder> page = advanceOrderService.page(new Page<>(current, pageSize), queryWrapper);
        // 封装成写真预约订单VO
        List<AdvanceOrder> records = page.getRecords();
        List<AdvanceOrderVO> advanceOrderVOS = records.stream().map(advanceOrder -> {
            AdvanceOrderVO advanceOrderVO = new AdvanceOrderVO();
            BeanUtils.copyProperties(advanceOrder, advanceOrderVO);
            return advanceOrderVO;
        }).toList();
        Page<AdvanceOrderVO> voPage = new Page<>();
        voPage.setRecords(advanceOrderVOS);
        voPage.setTotal(page.getTotal());
        voPage.setPages(page.getSize());
        voPage.setCurrent(page.getCurrent());
        voPage.setSize(page.getSize());
        return ResultUtils.success(voPage);
    }




    /**
     * 小程序端用户查询写真预约订单
     * @return  当前用户的订单列表
     */
    @PostMapping("/list/my")
    @Operation(summary = "小程序端用户查询写真预约订单", description = "参数：无, 权限：所有人，方法名：listMyAdvanceOrder")
    public BaseResponse<List<AdvanceOrderVO>> listMyAdvanceOrder(HttpServletRequest request) {
        User loginUser = userService.getLoginUser(request);
        Long userId = loginUser.getId();
        QueryWrapper<AdvanceOrder> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("userId", userId);
        List<AdvanceOrder> advanceOrderList = advanceOrderService.list(queryWrapper);
        // 封装成写真预约订单VO
        List<AdvanceOrderVO> advanceOrderVOS = advanceOrderList.stream().map(advanceOrder -> {
            AdvanceOrderVO advanceOrderVO = new AdvanceOrderVO();
            BeanUtils.copyProperties(advanceOrder, advanceOrderVO);
            return advanceOrderVO;
        }).toList();
        return ResultUtils.success(advanceOrderVOS);
    }


    /**
     * Web端管理员更新订单状态
     * @param orderUpdateRequest    订单状态更新请求体
     * @return 是否更新成功
     */
    @PostMapping("/update")
    @Operation(summary = "Web端管理员更新订单状态", description = "参数：订单状态更新请求体，权限：管理员(admin, boss)， 方法名：updateAdvanceOrderStatus")
    public BaseResponse<Boolean> updateAdvanceOrderStatus(@RequestBody OrderUpdateRequest orderUpdateRequest) {
        if (orderUpdateRequest == null || orderUpdateRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Long id = orderUpdateRequest.getId();
        String orderStatus = orderUpdateRequest.getOrderStatus();
        UpdateWrapper<AdvanceOrder> updateWrapper = new UpdateWrapper<>();
        updateWrapper.eq("id", id);
        updateWrapper.set("orderStatus", orderStatus);
        boolean result = advanceOrderService.update(updateWrapper);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR);
        return ResultUtils.success(true);
    }



}
