package com.cultural.heritage.controller.book;


import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.cultural.heritage.annotation.AuthCheck;
import com.cultural.heritage.common.BaseResponse;
import com.cultural.heritage.common.ErrorCode;
import com.cultural.heritage.common.ResultUtils;
import com.cultural.heritage.constant.UserConstant;
import com.cultural.heritage.exception.BusinessException;
import com.cultural.heritage.exception.ThrowUtils;
import com.cultural.heritage.model.dto.CommonRequest;
import com.cultural.heritage.model.dto.clothesinfo.ClothesInfoAddRequest;
import com.cultural.heritage.model.dto.clothesinfo.ClothesInfoQueryRequest;
import com.cultural.heritage.model.dto.clothesinfo.ClothesInfoUpdateRequest;
import com.cultural.heritage.model.entity.ClothesGrade;
import com.cultural.heritage.model.entity.ClothesInfo;
import com.cultural.heritage.model.vo.clothesinfo.ClothesInfoLabelVO;
import com.cultural.heritage.model.vo.clothesinfo.ClothesInfoVO;
import com.cultural.heritage.service.book.ClothesGradeService;
import com.cultural.heritage.service.book.ClothesInfoService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@RestController
@RequestMapping("/clothesInfo")
@Slf4j
@Tag(name = "服装信息管理模块")
public class ClothesInfoController {
    

    @Resource
    private ClothesInfoService clothesInfoService;


    @Resource
    private ClothesGradeService clothesGradeService;

    /**
     * 添加服装
     * @param clothesInfoAddRequest 服装添加请求体
     * @return  服装的id
     */
    @PostMapping("/add")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    @Operation(summary = "Web端管理员添加服装", description = "参数：服装添加请求体，权限：管理员(admin, boss)，方法名：addClothesInfo")
    public BaseResponse<Long> addClothesInfo(@RequestBody ClothesInfoAddRequest clothesInfoAddRequest) {
        if (clothesInfoAddRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        ClothesInfo clothesInfo = new ClothesInfo();
        BeanUtils.copyProperties(clothesInfoAddRequest, clothesInfo);
        // 校验
        clothesInfoService.validClothesInfo(clothesInfo, false);
        boolean result = clothesInfoService.save(clothesInfo);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR);
        return ResultUtils.success(clothesInfo.getId());
    }




    /**
     * 更新服装
     * @param clothesInfoUpdateRequest  服装更新请求体
     * @return 是否更新成功
     */
    @PostMapping("/update")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    @Operation(summary = "Web管理员更新服装", description = "参数：服装更新请求体，权限：管理员(admin, boss)，方法名：updateClothesInfo")
    public BaseResponse<Boolean> updateClothesInfo(@RequestBody ClothesInfoUpdateRequest clothesInfoUpdateRequest) {
        if (clothesInfoUpdateRequest == null || clothesInfoUpdateRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        ClothesInfo clothesInfo = new ClothesInfo();
        BeanUtils.copyProperties(clothesInfoUpdateRequest, clothesInfo);
        // 校验
        clothesInfoService.validClothesInfo(clothesInfo, true);
        boolean result = clothesInfoService.updateById(clothesInfo);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR);
        return ResultUtils.success(true);
    }


    /**
     * 删除服装
     * @param commonRequest  服装删除请求体
     * @return  是否删除成功
     */
    @PostMapping("/delete")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    @Operation(summary = "Web管理员删除服装", description = "参数：服装删除请求体，权限：管理员(admin, boss)，方法名：deleteClothesInfo")
    public BaseResponse<Boolean> deleteClothesInfo(@RequestBody CommonRequest commonRequest) {
        if (commonRequest == null || commonRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Long id = commonRequest.getId();
        boolean result = clothesInfoService.removeById(id);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR);
        return ResultUtils.success(true);
    }



    /**
     * 分页查询服装列表
     * @param clothesInfoQueryRequest   服装查询请求体
     * @return  服装列表
     */
    @PostMapping("/list/page")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    @Operation(summary = "Web管理员分页查询服装", description = "参数：服装删除请求体，权限：管理员(admin, boss)，方法名：deleteClothesInfo")
    public BaseResponse<Page<ClothesInfoVO>> listClothesInfoByPage(@RequestBody ClothesInfoQueryRequest clothesInfoQueryRequest) {
        if (clothesInfoQueryRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        long current = clothesInfoQueryRequest.getCurrent();
        long pageSize = clothesInfoQueryRequest.getPageSize();
        QueryWrapper<ClothesInfo> queryWrapper = clothesInfoService.getQueryWrapper(clothesInfoQueryRequest);
        Page<ClothesInfo> page = clothesInfoService.page(new Page<>(current, pageSize), queryWrapper);
        // 封装服装类VO
        List<ClothesInfo> records = page.getRecords();
        List<ClothesInfoVO> clothesInfoVOS = records.stream().map(clothesInfo -> {
            ClothesInfoVO clothesInfoVO = new ClothesInfoVO();
            BeanUtils.copyProperties(clothesInfo, clothesInfoVO);
            return clothesInfoVO;
        }).toList();
        Page<ClothesInfoVO> voPage = new Page<>();
        voPage.setRecords(clothesInfoVOS);
        voPage.setTotal(page.getTotal());
        voPage.setCurrent(page.getCurrent());
        voPage.setSize(page.getSize());
        voPage.setPages(page.getPages());
        return ResultUtils.success(voPage);
    }




    /**
     * 小程序端用户根据服装等级id查询服装标签
     * @param commonRequest 服装等级id
     * @return  当前等级下的服装列表标签
     */
    @PostMapping("/list/label")
    @Operation(summary = "小程序端用户根据服装等级id查询服装标签", description = "参数：服装等级id，权限：所有人，方法名：listClothesInfoLabel")
    public BaseResponse<List<ClothesInfoLabelVO>> listClothesInfoLabel(@RequestBody CommonRequest commonRequest) {
        if (commonRequest == null || commonRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Long id = commonRequest.getId();
        ClothesGrade clothesGrade = clothesGradeService.getById(id);
        String clothesType = clothesGrade.getClothesType();
        QueryWrapper<ClothesInfo> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("clothesType", clothesType);
        List<ClothesInfo> clothesInfoList = clothesInfoService.list(queryWrapper);

        List<ClothesInfoLabelVO> clothesInfoLabelVOS = clothesInfoList.stream().map(clothesInfo -> {
            ClothesInfoLabelVO clothesInfoLabelVO = new ClothesInfoLabelVO();
            BeanUtils.copyProperties(clothesInfo, clothesInfoLabelVO);
            return clothesInfoLabelVO;
        }).toList();
        return ResultUtils.success(clothesInfoLabelVOS);
    }



    /**
     * 小程序端用户根据服装等级id查询服装详情
     * @param commonRequest 服装id
     * @return  服装详情信息
     */
    @PostMapping("/list/detail")
    @Operation(summary = "小程序端用户根据服装等级id查询服装详情", description = "参数：服装id，权限：所有人，方法名：getOneById")
    public BaseResponse<ClothesInfoVO> getOneById(@RequestBody CommonRequest commonRequest) {
        if (commonRequest == null || commonRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Long id = commonRequest.getId();
        ClothesInfo clothesInfo = clothesInfoService.getById(id);
        ThrowUtils.throwIf(clothesInfo == null, ErrorCode.NOT_FOUND_ERROR, "该服装已被删除");
        ClothesInfoVO clothesInfoVO = new ClothesInfoVO();
        BeanUtils.copyProperties(clothesInfo, clothesInfoVO);
        return ResultUtils.success(clothesInfoVO);
    }




}
