package com.cultural.heritage.service.book.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.cultural.heritage.common.ErrorCode;
import com.cultural.heritage.constant.BookConstant;
import com.cultural.heritage.constant.CommonConstant;
import com.cultural.heritage.exception.BusinessException;
import com.cultural.heritage.mapper.AdvanceOrderMapper;
import com.cultural.heritage.model.dto.advanceOrder.AdvanceOrderQueryRequest;
import com.cultural.heritage.model.dto.snapshot.ContactsSnapshot;
import com.cultural.heritage.model.entity.AdvanceOrder;
import com.cultural.heritage.service.book.AdvanceOrderService;
import com.cultural.heritage.utils.SqlUtils;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;

@Service
public class AdvanceOrderServiceImpl extends ServiceImpl<AdvanceOrderMapper, AdvanceOrder> implements AdvanceOrderService {


    /**
     * 校验
     */
    @Override
    public void validAdvanceOrder(AdvanceOrder advanceOrder) {
        String type = advanceOrder.getType();
        String clothesType = advanceOrder.getClothesType();
        BigDecimal deposit = advanceOrder.getDeposit();
        ContactsSnapshot contactsSnapshot = advanceOrder.getContactsSnapshot();
        String specificDate = advanceOrder.getSpecificDate();
        String timeSlot = advanceOrder.getTimeSlot();
        Integer isIndoors = advanceOrder.getIsIndoors();
        Integer isMakeup = advanceOrder.getIsMakeup();
        Integer isPhotography = advanceOrder.getIsPhotography();
        String orderStatus = advanceOrder.getOrderStatus();
        Long userId = advanceOrder.getUserId();

        if (ObjectUtils.isEmpty(deposit) || deposit.compareTo(BigDecimal.ZERO) <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "定金参数错误");
        }
        if (ObjectUtils.anyNull(contactsSnapshot, isIndoors, isMakeup, isPhotography, userId)) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "存在参数为null");
        }
        if (isIndoors != 0 && isIndoors != 1 || isMakeup != 0 && isMakeup != 1 || isPhotography != 0 && isPhotography != 1) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "布尔参数错误");
        }
        if (StringUtils.isAnyBlank(type, clothesType, specificDate, timeSlot, orderStatus)) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "存在参数为空");
        }
        if (!type.equals(BookConstant.RENT_CLOTHES) && !type.equals(BookConstant.OWN_CLOTHES)) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "预约类型参数错误");
        }
        if (type.equals(BookConstant.RENT_CLOTHES)) {
            if (isMakeup != 1 || isPhotography != 1) {
                throw new BusinessException(ErrorCode.PARAMS_ERROR, "是否妆发或是否摄影参数错误");
            }
        }
    }


    /**
     * 获取查询条件
     */
    @Override
    public QueryWrapper<AdvanceOrder> getQueryWrapper(AdvanceOrderQueryRequest advanceOrderQueryRequest) {
        Long id = advanceOrderQueryRequest.getId();
        String type = advanceOrderQueryRequest.getType();
        BigDecimal minDeposit = advanceOrderQueryRequest.getMinDeposit();
        BigDecimal maxDeposit = advanceOrderQueryRequest.getMaxDeposit();
        String startTime = advanceOrderQueryRequest.getStartTime();
        String endTime = advanceOrderQueryRequest.getEndTime();
        String orderStatus = advanceOrderQueryRequest.getOrderStatus();
        String sortField = advanceOrderQueryRequest.getSortField();
        String sortOrder = advanceOrderQueryRequest.getSortOrder();
        String createLeftTime = advanceOrderQueryRequest.getCreateLeftTime();
        String createRightTime = advanceOrderQueryRequest.getCreateRightTime();

        QueryWrapper<AdvanceOrder> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(id != null, "id", id);
        queryWrapper.eq(StringUtils.isNotBlank(type), "type", type);
        queryWrapper.ge(ObjectUtils.isNotEmpty(minDeposit), "deposit", minDeposit);
        queryWrapper.le(ObjectUtils.isNotEmpty(maxDeposit), "deposit", maxDeposit);
        queryWrapper.between(StringUtils.isNotBlank(startTime) && StringUtils.isNotBlank(endTime), "specificDate", startTime, endTime);
        queryWrapper.between(StringUtils.isNotBlank(createLeftTime) && StringUtils.isNotBlank(createRightTime), "createTime", createLeftTime, createRightTime);
        queryWrapper.eq(StringUtils.isNotBlank(orderStatus), "orderStatus", orderStatus);
        queryWrapper.orderBy(SqlUtils.validSortField(sortField), sortOrder.equals(CommonConstant.SORT_ORDER_ASC),
                sortField);
        return queryWrapper;
    }
}
