package com.cultural.heritage.service.good.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.cultural.heritage.common.ErrorCode;
import com.cultural.heritage.constant.CommonConstant;
import com.cultural.heritage.exception.BusinessException;
import com.cultural.heritage.mapper.GoodMapper;
import com.cultural.heritage.model.dto.good.GoodQueryRequest;
import com.cultural.heritage.model.entity.Good;
import com.cultural.heritage.service.good.GoodService;
import com.cultural.heritage.utils.SqlUtils;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

@Service
public class GoodServiceImpl extends ServiceImpl<GoodMapper, Good> implements GoodService {

    /**
     * 商品查询条件
     */
    @Override
    public QueryWrapper<Good> getGoodQueryWrapper(GoodQueryRequest goodQueryRequest) {
        if (goodQueryRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "请求参数为空");
        }
        Long id = goodQueryRequest.getId();
        String name = goodQueryRequest.getName();
        String type = goodQueryRequest.getType();
        Integer festivalOrder = goodQueryRequest.getFestivalOrder();
        Integer isShelves = goodQueryRequest.getIsShelves();
        String sortField = goodQueryRequest.getSortField();
        String sortOrder = goodQueryRequest.getSortOrder();

        QueryWrapper<Good> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(ObjectUtils.isNotEmpty(id), "id", id);
        queryWrapper.like(StringUtils.isNotBlank(name), "name", name);
        queryWrapper.like(StringUtils.isNotBlank(type), "type", type);
        queryWrapper.eq(ObjectUtils.isNotEmpty(festivalOrder), "festivalOrder", festivalOrder);
        queryWrapper.eq(ObjectUtils.isNotEmpty(isShelves), "isShelves", isShelves);
        queryWrapper.orderBy(SqlUtils.validSortField(sortField), sortOrder.equals(CommonConstant.SORT_ORDER_ASC),
                sortField);
        return queryWrapper;
    }


    /**
     * 根据商品类名查询商品列表
     */
    @Override
    public List<Good> getGoodListByTypeName(String typeName) {
        QueryWrapper<Good> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("type", typeName);
        List<Good> list = this.list(queryWrapper);
        if (CollectionUtils.isEmpty(list)) {
            return new ArrayList<>();
        }
        return list;
    }


    /**
     * 校验
     */
    @Override
    public void validGood(Good good, boolean update) {
        Long id = good.getId();
        String name = good.getName();
        String type = good.getType();
        String goodImg = good.getGoodImg();
        String intro = good.getIntro();
        String introDetail = good.getIntroDetail();
        String detailImg = good.getDetailImg();
        String label = good.getLabel();
        Integer inventory = good.getInventory();
        Integer festivalOrder = good.getFestivalOrder();
        BigDecimal price = good.getPrice();

        if (update) {
            if (id == null) {
                throw new BusinessException(ErrorCode.PARAMS_ERROR, "参数id错误");
            }
        }
        if (StringUtils.isAnyBlank(type, goodImg, intro, introDetail, detailImg, label, name)) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "存在参数为空");
        }
        if (ObjectUtils.isEmpty(inventory) || inventory < 1) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "库存量不能小于1");
        }
        if (ObjectUtils.isEmpty(festivalOrder) || festivalOrder <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "节日参数错误");
        }
        if (ObjectUtils.isEmpty(price) || price.compareTo(BigDecimal.ZERO) <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "商品价格不能小于等于0");
        }


    }

}
