package com.cultural.heritage.service.order.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.cultural.heritage.common.ErrorCode;
import com.cultural.heritage.constant.CommonConstant;
import com.cultural.heritage.exception.BusinessException;
import com.cultural.heritage.exception.ThrowUtils;
import com.cultural.heritage.mapper.*;
import com.cultural.heritage.model.dto.order.OrderAddRequest;
import com.cultural.heritage.model.dto.order.OrderItemAddRequest;
import com.cultural.heritage.model.dto.order.OrderQueryRequest;
import com.cultural.heritage.model.dto.order.capital.OrderItemMainInfoAddRequest;
import com.cultural.heritage.model.dto.order.capital.OrderMainInfoAddRequest;
import com.cultural.heritage.model.dto.snapshot.AddressSnapshot;
import com.cultural.heritage.model.dto.snapshot.ContactsSnapshot;
import com.cultural.heritage.model.dto.snapshot.CouponSnapshot;
import com.cultural.heritage.model.dto.snapshot.GoodSnapshot;
import com.cultural.heritage.model.entity.Order;
import com.cultural.heritage.model.enums.GoodTypeEnum;
import com.cultural.heritage.service.order.OrderService;
import com.cultural.heritage.utils.SqlUtils;
import jakarta.annotation.Resource;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.Date;
import java.util.List;

@Service
public class OrderServiceImpl extends ServiceImpl<OrderMapper, Order> implements OrderService {


    @Resource
    private AddressMapper addressMapper;


    @Resource
    private ContactsMapper contactsMapper;


    @Resource
    private CouponMapper couponMapper;


    @Resource
    private GoodMapper goodMapper;



    /**
     * 获取查询条件
     */
    @Override
    public QueryWrapper<Order> getQueryWrapper(OrderQueryRequest orderQueryRequest) {
        if (orderQueryRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Long userId = orderQueryRequest.getUserId();
        Long orderId = orderQueryRequest.getOrderId();
        BigDecimal minTotalAmount = orderQueryRequest.getMinTotalAmount();
        BigDecimal maxTotalAmount = orderQueryRequest.getMaxTotalAmount();
        String orderStatus = orderQueryRequest.getOrderStatus();
        String orderNumber = orderQueryRequest.getOrderNumber();
        Date startTime = orderQueryRequest.getStartTime();
        Date endTime = orderQueryRequest.getEndTime();
        String sortOrder = orderQueryRequest.getSortOrder();
        String sortField = orderQueryRequest.getSortField();
        String orderType = orderQueryRequest.getOrderType();

        QueryWrapper<Order> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(ObjectUtils.isNotEmpty(userId), "userId", userId);
        queryWrapper.eq(ObjectUtils.isNotEmpty(orderId), "id", orderId);
        queryWrapper.ge(ObjectUtils.isNotEmpty(minTotalAmount), "totalAmount", minTotalAmount);
        queryWrapper.le(ObjectUtils.isNotEmpty(maxTotalAmount), "totalAmount", maxTotalAmount);
        queryWrapper.eq(StringUtils.isNotBlank(orderStatus), "orderStatus", orderStatus);
        queryWrapper.eq(StringUtils.isNotBlank(orderType), "orderType", orderType);
        queryWrapper.between(startTime != null && endTime != null, "createTime", startTime, endTime);
        queryWrapper.eq(StringUtils.isNotBlank(orderNumber), "orderNumber", orderNumber);
        queryWrapper.orderBy(SqlUtils.validSortField(sortField), sortOrder.equals(CommonConstant.SORT_ORDER_ASC),
                sortField);
        return queryWrapper;
    }


    /**
     * 封装订单详细信息请求体
     */
    @Override
    public OrderAddRequest initOrderMainInfo(OrderMainInfoAddRequest orderMainInfoAddRequest) {
        String userName = orderMainInfoAddRequest.getUserName();
        Long addressId = orderMainInfoAddRequest.getAddressId();
        Long contactsId = orderMainInfoAddRequest.getContactsId();
        Long couponId = orderMainInfoAddRequest.getCouponId();
        String orderType = orderMainInfoAddRequest.getOrderType();
        String orderNumber = orderMainInfoAddRequest.getOrderNumber();
        String orderStatus = orderMainInfoAddRequest.getOrderStatus();
        String note = orderMainInfoAddRequest.getNote();
        BigDecimal totalAmount = orderMainInfoAddRequest.getTotalAmount();
        List<OrderItemMainInfoAddRequest> orderItemMainInfoAddRequestList = orderMainInfoAddRequest.getOrderItemMainInfoAddRequestList();


        GoodTypeEnum orderTypeEnum = GoodTypeEnum.getEnumByValue(orderType);
        ThrowUtils.throwIf(orderTypeEnum == null, ErrorCode.PARAMS_ERROR);
        AddressSnapshot addressSnapshot = null;
        ContactsSnapshot contactsSnapshot = null;
        CouponSnapshot couponSnapshot = null;

        // 获取订单地址信息
        if (orderTypeEnum.equals(GoodTypeEnum.PRODUCT)) {
            addressSnapshot = getSnapshot(addressId, addressMapper, AddressSnapshot.class);
        }
        // 获取订单联系人信息
        if (orderTypeEnum.equals(GoodTypeEnum.SERVICE)) {
            contactsSnapshot = getSnapshot(contactsId, contactsMapper, ContactsSnapshot.class);
        }
        // 获取订单优惠券信息
        if (couponId != null) {
            couponSnapshot = getSnapshot(couponId, couponMapper, CouponSnapshot.class);
        }
        // 封装订单明细信息
        List<OrderItemAddRequest> orderItemAddRequestList = orderItemMainInfoAddRequestList.stream().map(orderItemMainInfoAddRequest -> {
            Long goodId = orderItemMainInfoAddRequest.getGoodId();
            Integer quantity = orderItemMainInfoAddRequest.getQuantity();
            String reservationDate = orderItemMainInfoAddRequest.getReservationDate();
            String timeSlot = orderItemMainInfoAddRequest.getTimeSlot();
            OrderItemAddRequest orderItemAddRequest = new OrderItemAddRequest();
            // 获取订单商品信息
            GoodSnapshot goodSnapshot = getSnapshot(goodId, goodMapper, GoodSnapshot.class);

            orderItemAddRequest.setGoodSnapshot(goodSnapshot);
            orderItemAddRequest.setPriceSnapshot(goodSnapshot.getPrice());
            orderItemAddRequest.setQuantity(quantity);
            orderItemAddRequest.setItemTotalAmount(goodSnapshot.getPrice().multiply(BigDecimal.valueOf(quantity)));
            orderItemAddRequest.setReservationDate(reservationDate);
            orderItemAddRequest.setTimeSlot(timeSlot);
            return orderItemAddRequest;
        }).toList();

        // 创建订单详细信息请求体
        OrderAddRequest orderAddRequest = new OrderAddRequest();
        orderAddRequest.setUserName(userName);
        orderAddRequest.setOrderNumber(orderNumber);
        orderAddRequest.setOrderStatus(orderStatus);
        orderAddRequest.setOrderType(orderType);
        orderAddRequest.setNote(note);
        orderAddRequest.setTotalAmount(totalAmount);
        orderAddRequest.setAddressSnapshot(addressSnapshot);
        orderAddRequest.setContactsSnapshot(contactsSnapshot);
        orderAddRequest.setCouponSnapshot(couponSnapshot);
        orderAddRequest.setOrderItemList(orderItemAddRequestList);
        return orderAddRequest;
    }



    /**
     * 校验订单
     */
    @Override
    public void validOrder(Order order) {
        String orderType = order.getOrderType();
        String orderNumber = order.getOrderNumber();
        Long userId = order.getUserId();
        String userName = order.getUserName();
        BigDecimal totalAmount = order.getTotalAmount();
        String orderStatus = order.getOrderStatus();
        if (ObjectUtils.isEmpty(totalAmount) || totalAmount.compareTo(BigDecimal.ZERO) <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "价格错误");
        }
        if (ObjectUtils.isEmpty(userId)) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        if (StringUtils.isAnyBlank(orderType, orderNumber, userName, orderStatus)) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "参数不全");
        }
    }



    /**
     * 根据id获取订单(地址，联系人，优惠券，商品)信息
     */
    private static <T, S> S getSnapshot(Long id, BaseMapper<T> mapper, Class<S> snapshotClass) {
        // 创建查询条件
        QueryWrapper<T> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("id", id);

        // 查询实体对象
        T entity = mapper.selectOne(queryWrapper);

        // 判断实体对象是否存在，不存在则抛出异常
        ThrowUtils.throwIf(entity == null, ErrorCode.NOT_FOUND_ERROR);

        try {
            // 创建快照对象实例
            S snapshot = snapshotClass.getDeclaredConstructor().newInstance();

            // 复制属性到快照对象
            BeanUtils.copyProperties(entity, snapshot);

            return snapshot;
        } catch (Exception e) {
            throw new BusinessException(ErrorCode.SYSTEM_ERROR);
        }
    }



}
